//Displays temperature of ds18B20 sensor. Draws thermometer as a bitmap from const char table.
//Draws a plot of temperature

#define DIR 2
#include "thermometer_bmp.h"
#include <OSD50HD.h>

#include <OneWire.h>
#include <DallasTemperature.h>

OneWire  ds(7);

OSD50HD osd;
void setup() {
  delay(8000); //8000

  Serial.begin (115200);
  pinMode(DIR, OUTPUT);
  digitalWrite(DIR, LOW);

  delay(40);
  osd.SendGrid(1, 25, 76, 386, 256, 1, 40, 20, 1, 30); //Net settings
  delay(40);
  osd.SendLineHV(1, 20, 70, 376, 230, 2, 0, 15); //Net background
  delay(40);
  osd.SetPlot(1, 24, 85, 22, 20, 3, 1, 28); //Plot settings
  delay(40);
  osd.SendText(1, 2, 9, "Temperature plot");

  osd.SendCircle(1, 80, 610, 70, 75, 75, 2, 0, 8);//background of thermometer
  delay(40);
  osd.SendLineHV(1, 35, 600, 120, 430, 2, 0, 12); //background of thermometer
  delay(40);

  osd.SetBitmap(1, 40, 585, 255, 255, 2 , 0, 1, 38);
  osd.UpdateBitmap(1, 0, 0, 5112, thermometer_bmp);
}
void loop() {
  TemperatureMeasure();
}

void TemperatureMeasure(void)
{
  static byte i, conversion = 0;
  static byte present = 0;
  static  byte data[12];
  static byte addr[8];
  static unsigned char tab[20];
  static uint32_t tit = 0, TWT = 0;
  if (millis() - tit > TWT)
  {
    if (conversion == 0)
    {
      ds.search(addr);
      ds.reset_search();
      if ( OneWire::crc8( addr, 7) != addr[7]) {
        return;
      }

      ds.reset();
      ds.select(addr);
      ds.write(0x44, 1);        // start conversion
      conversion = 1;
      TWT = 800;
      tit = millis();
    }
    else if (conversion == 1)
    {
      present = ds.reset();
      ds.select(addr);
      ds.write(0xBE);         // Read Scratchpad

      for ( i = 0; i < 9; i++) {           // we need 9 bytes
        data[i] = ds.read();
      }
      int HighByte, LowByte;
      LowByte = data[0];
      HighByte = data[1];
      float t = (HighByte << 8) + LowByte;
      t /= 16;
      char tmp[10];
      dtostrf(t, 1, 2, tmp);
      osd.SendText(1, 1, 22, tmp);
      int tt = - t * 4;

      osd.SendLineHV(1, 64, 833 + tt, 10, 135 - tt, 1, 0, 14); //temperature bar
      tt *= 4;
      tt += 550;
      if (tt < 0)
        tt = 0;
      if (tt > 253)
        tt = 253;
      osd.UpdatePlot(1, 0, tt);
      conversion = 0;
      TWT = 10;
      tit = millis();
    }
  }
}
